(require 'flange-util)

(defvar phalange-timeouts '())

(defun phalange-timeout-name (to) (car to))
(defun phalange-timeout-time (to) (cdr to))

(defun phalange-timer< (l r)
  (let ((l-time (phalange-timeout-time l))
	(l-name (phalange-timeout-name l))
	(r-time (phalange-timeout-time r))
	(r-name (phalange-timeout-name r)))
    (or (and r-time (not l-time))
	(and (not (or l-time r-time))
	     (string< (symbol-name l-name)
		      (symbol-name r-name)))
	(and l-time
	     r-time
	     (time< l-time r-time)))
    ))

(defun phalange-make-timeout (name)
  (let ((timeout (assq (symbol-concat 'phalange name) phalange-timeouts)))
    (and timeout
	 (error "Timer %S already exists" name))
    (setq phalange-timeouts
	  (sort (cons (setq timeout (cons name nil))
		      phalange-timeouts)
		'phalange-timer<
		))
    timeout))


(defun set-timeout (timer n &optional units)
  (let ((timer (assq timer phalange-timeouts)))
    (if (not timer)
	(progn
	  (cerror "Timer %S does not exist" timer)
	  (setq timer (phalange-make-timeout timer)))
      (let* ((delta (as-time n units))
	     (expire-time (time+ delta (current-time))))
	(setcdr timer expire-time)))
    ))


(defun stop-timeout (timer)
  (let ((timer (assq timer phalange-timeouts)))
    (if timer
	(progn
	  (setcdr timer nil)
	  (setq phalange-timeouts (sort phalange-timeouts 'phalange-timer<))
	  timer))
    ))


(defun phalange-timer-pending ()
  (let ((now (current-time))
	(next-timeout (and phalange-timeouts
			   (car phalange-timeouts))))
    (and next-timeout
	 (let ((next (phalange-timeout-time next-timeout)))
	   (and next
		(time< next now)))
	 next-timeout)))


(provide 'flange-timers)