;;; alternate name tenson, punning on tendon and tension, and 
;;; taking care of the 'ten sons' which are the fingers...

(defun symbol-concat (s1 s2)
  (intern
   (concat (symbol-name s1)
	   "-"
	   (symbol-name s2))))


(defun time< (l r)
  "compares values returned by (current-time)"
  (let ((l-high (car l)) (l-low (cadr l)) (l-micro (caddr l))
	(r-high (car r)) (r-low (cadr r)) (r-micro (caddr r)))
    (or (< l-high r-high)
	(and (= l-high r-high)
	     (or (< l-low r-low)
		 (and (= l-low r-low)
		      (< l-micro r-micro)))))
    ))


(defun addc (overflow &rest args)
  (let ((s (apply '+ args)))
    (cons (mod s overflow) (floor s overflow))))


(defun time+ (l r)
  (let ((l-high (car l)) (l-low (cadr l)) (l-micro (caddr l))
	(r-high (car r)) (r-low (cadr r)) (r-micro (caddr r)))
    (let* ((s-micro (addc 1000000 l-micro r-micro))
	   (s-low (addc 65536 (cdr s-micro) l-low r-low))
	   (s-high (+ (cdr s-low) l-high r-high)))
      (list s-high (car s-low) (car s-micro))
      )))


(defconst time-units
  '((minute 60)
    (minutes 60)
    (hour 3600)
    (hours 3600)
    (day 86400)
    (days 86400)
    (second 1)
    (seconds 1)
    (fortnight 1209600)   ; for VMS-heads
    (fortnights 1209600)
    )
  "multipliers for conversion to seconds")


(defun as-seconds (n units)
  (let ((seconds (assq units time-units)))
    (if seconds
	(* n (cadr seconds))
      n)))


(defun as-time (n units)
  (let* ((seconds (as-seconds n units))
	 (micro (floor (* (- seconds (floor seconds)) 1000000)))
	 (low (mod (floor seconds) 65536))
	 (high (floor seconds 65536)))
    (list high low micro)))


(defun calculate-delta-time (n units)
  (let ((now (current-time)))
    (time+ now (as-time n units))))


(provide 'flange-util)