;;; alternate name tenson, punning on tendon and tension, and 
;;; taking care of the 'ten sons' which are the fingers...

(require 'flange-util)
(require 'flange-timers)
(require 'flange-dispatch)

(defun dephalange-error (state-name message)
  "report errors in phalange state machine forms"
  (error "Error defining state %S: %s" state-name message))


(defun phalange-build-state-definition (state-name event-handlers)
  "builds a phalange state"
  (let ((cond-clauses '())
	(epsilon-transition-disallowed (> (length event-handlers) 1)))
    (while event-handlers
      (let* ((handler (car event-handlers))
	     (events (car handler))
	     (body (cdr handler)))
	(setq event-handlers (cdr event-handlers))
	(setq cond-clauses
	      (if epsilon-transition-disallowed
		  (if (not events) 
		      (dephalange-error state-name
					"Empty event set handles *all* events")
		    (cons `((memq phalange-event ',events)
			    ,@body)
			  cond-clauses))
		`((t ,@body))))
	))
    `(defun ,(symbol-concat 'phalange-state state-name) (phalange-event)
       (cond ,@(reverse cond-clauses)))
    ))


(defmacro dephalange (state-name &rest event-handlers)
  (phalange-build-state-definition state-name event-handlers))

(put 'dephalange 'lisp-indent-function 2)
(put 'dephalange
     'edebug-form-spec
     '(&define :name 'phalange-state- name def-body))


(defun phalange-post-command (&optional hilight)
  (let ((now (current-time)))
    (and (phalange-timer-pending)
	 (phalange-dispatch-timeouts))
    (phalange-dispatch-command this-command)
    ))


(add-hook 'post-command-hook 'phalange-post-command)
